#pragma once

#include <mutex>
#include <optional>

#include <Eigen/Core>

#include <ArmarXCore/core/application/properties/PropertyDefinitionContainer.h>

#include <RobotAPI/libraries/armem/client/MemoryNameSystem.h>
#include <RobotAPI/libraries/armem/client/Reader.h>
#include <RobotAPI/libraries/armem_locations/aron/Location.aron.generated.h>
#include <RobotAPI/libraries/armem_objects/client/instance/ObjectReader.h>
#include <RobotAPI/libraries/armem_robot_state/client/common/RobotReader.h>

namespace armarx::armem::locations::client
{
    class Reader
    {
    public:
        struct Properties
        {
            std::string memoryName = "Navigation";
        };

        Reader(armem::client::MemoryNameSystem& memoryNameSystem);
        virtual ~Reader() = default;

        void registerPropertyDefinitions(armarx::PropertyDefinitionsPtr& def);
        void connect();

        Properties
        getProperties()
        {
            return this->p;
        }

        std::map<std::string, armarx::navigation::location::arondto::Location> getAllLocations();
        std::map<std::string, armarx::navigation::location::arondto::Location>
        getAllLocationsInGlobalFrame();

    private:
        Properties p;

        const std::string propertyPrefix = "mem.nav.location";

        armem::client::MemoryNameSystem& memoryNameSystem;
        armarx::armem::obj::instance::Reader objReader;
        armarx::armem::robot_state::RobotReader robotReader;

        armarx::armem::client::Reader locationReader;
    };

} // namespace armarx::armem::locations::client
